@@ -2,11 +2,13 @@ from __future__ import absolute_import
 
 import time
 import string
+from contextlib import contextmanager
+
 import xapian
 
 from operator import itemgetter
 from functools import partial
-from json import dumps, loads
+from json import loads
 
 from xapian import Query, QueryParser, UnimplementedError, DocNotFoundError
 
@@ -168,6 +170,15 @@ class Database(object):
     value_count_name = "_XODB_COUNT_"
     backend = None
 
+    @contextmanager
+    def transaction(self):
+        self.begin()
+        try:
+            yield self
+            self.commit()
+        except:
+            self.cancel()
+
     def __init__(self, path,
                  writable=True,
                  overwrite=False,
@@ -326,7 +337,7 @@ class Database(object):
             added.append(doc)
         return added
 
-    def to_document(self, obj, validate=True):
+    def to_schema(self, obj, validate=True):
         if hasattr(obj, '__xodb_schema__'):
             schema_type = obj.__xodb_schema__
         else:
@@ -343,9 +354,11 @@ class Database(object):
                                   (schema.__class__.__name__,
                                    list((c.name, c.value)
                                         for c in invalid)))
+        return schema
 
-        doc = self.doc_from_dict(schema.memo.dict)
-        return doc
+    def to_document(self, obj, validate=True):
+        schema = self.to_schema(obj, validate)
+        return self.doc_from_dict(schema.memo.dict)
 
     def doc_from_dict(self, data):
         """
@@ -376,7 +389,12 @@ class Database(object):
             tg.set_document(doc)
             spelling = text_dict.get('spell', True)
             if spelling:
-                tg.set_flags(xapian.TermGenerator.FLAG_SPELLING)
+                try:
+                    # hack to workaround missing spelling for inmem backends
+                    self.backend.add_spelling('food')
+                    tg.set_flags(xapian.TermGenerator.FLAG_SPELLING)
+                except:
+                    pass # noop for backends that don't support spelling
             if lang in snowball.stoppers:
                 tg.set_stemmer(xapian.Stem(lang))
                 tg.set_stopper(snowball.stoppers[lang])
@@ -462,15 +480,24 @@ class Database(object):
     def begin(self):
         if self._writable:
             self.backend.reopen()
-            self.backend.begin_transaction()
+            try:
+                self.backend.begin_transaction()
+            except:
+                pass # noop for backends that don't support transactions
 
     def cancel(self):
         if self._writable:
-            self.backend.cancel_transaction()
+            try:
+                self.backend.cancel_transaction()
+            except:
+                pass
 
     def commit(self):
         if self._writable:
-            self.backend.commit_transaction()
+            try:
+                self.backend.commit_transaction()
+            except:
+                pass
 
     def prepare_query_parser(self, language=None,
                              default_op=Query.OP_AND):
