@@ -24,6 +24,7 @@ def _prefix(prefix, value=None):
         return
     return ("%s:%s" % (prefix, value)) if value else prefix
 
+
 def _normalize(value):
     return unicodedata.normalize(
         'NFKC', unicode(value).strip().lower()).encode('utf-8')
@@ -118,21 +119,25 @@ class Schema(schema.Form):
 
                 if value is not None:
                     if el.facet:
-                        self._memo.add_term(
-                            _prefix(self.facet_prefix, el.name),
-                            True)
+                        if el.name:
+                            term = el.name.lower()
+                            self._memo.add_term(
+                                _prefix(self.facet_prefix, term),
+                                True)
         return True
 
     def _handle_scalar(self, term, value, element, type=None):
         name = element.flattened_name()
         memo = self._memo
-        if element.prefix:
-            memo.add_term(_prefix(name, term), element.boolean)
-        else:
-            memo.add_term(term, element.boolean)
-        if element.sortable:
-            memo.add_value(name, value, type)
-        return value
+        if term:
+            term = term.lower()
+            if element.prefix:
+                memo.add_term(_prefix(name, term), element.boolean)
+            else:
+                memo.add_term(term, element.boolean)
+            if element.sortable:
+                memo.add_value(name, value, type)
+            return value
 
     def _handle_string(self, element):
         term = element.u
@@ -183,11 +188,13 @@ class Schema(schema.Form):
             value = value.encode(element.translit)
 
         if element.string:
-            if element.string_prefix:
-                memo.add_term(_prefix(element.string_prefix, value),
-                               element.boolean)
-            else:
-                memo.add_term(value)
+            if value:
+                term = value.lower()
+                if element.string_prefix:
+                    memo.add_term(_prefix(element.string_prefix, term),
+                                   element.boolean)
+                else:
+                    memo.add_term(term)
 
         if element.sortable:
             memo.add_value(name, value, 'string')
@@ -220,6 +227,7 @@ class Schema(schema.Form):
 
         for i in xrange(minv, maxv, step):
             val = "%s_%s" % (i, i + step)
+            val = val.lower()
             self._memo.add_term(_prefix(element.name, val), True)
         return True
 
